@@ -13,12 +13,9 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: dvdnav.h 1135 2008-09-06 21:55:51Z rathann $
- *
+ * You should have received a copy of the GNU General Public License along
+ * with libdvdnav; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
@@ -26,22 +23,20 @@
  * to access dvdnav functionality.
  */
 
-#ifndef DVDNAV_H_INCLUDED
-#define DVDNAV_H_INCLUDED
+#ifndef LIBDVDNAV_DVDNAV_H
+#define LIBDVDNAV_DVDNAV_H
 
-#define MP_DVDNAV 1
+#define DVDNAV_VERSION 50400
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef DVDNAV_COMPILE
-#  include <dvdnav/dvd_types.h>
-#  include <dvdread/dvd_reader.h>
-#  include <dvdread/nav_types.h>
-#  include <dvdread/ifo_types.h> /* For vm_cmd_t */
-#  include <dvdnav/dvdnav_events.h>
-#endif
+#include <dvdnav/dvd_types.h>
+#include <dvdread/dvd_reader.h>
+#include <dvdread/nav_types.h>
+#include <dvdread/ifo_types.h> /* For vm_cmd_t */
+#include <dvdnav/dvdnav_events.h>
 
 
 
@@ -60,6 +55,8 @@ typedef struct dvdnav_s dvdnav_t;
 /* Status as reported by most of libdvdnav's functions */
 typedef int32_t dvdnav_status_t;
 
+typedef dvd_reader_stream_cb dvdnav_stream_cb;
+
 /*
  * Unless otherwise stated, all functions return DVDNAV_STATUS_OK if
  * they succeeded, otherwise DVDNAV_STATUS_ERR is returned and the error may
@@ -68,12 +65,6 @@ typedef int32_t dvdnav_status_t;
 #define DVDNAV_STATUS_ERR 0
 #define DVDNAV_STATUS_OK  1
 
-#define DVDNAV_FORMAT_AC3 0
-#define DVDNAV_FORMAT_MPEGAUDIO 3
-#define DVDNAV_FORMAT_LPCM 4
-#define DVDNAV_FORMAT_DTS 5
-#define DVDNAV_FORMAT_SDDS 6
-
 /*********************************************************************
  * initialisation & housekeeping functions                           *
  *********************************************************************/
@@ -84,15 +75,22 @@ typedef int32_t dvdnav_status_t;
  */
 
 /*
- * Attempts to open the DVD drive at the specified path and pre-cache
- * the CSS-keys. libdvdread is used to access the DVD, so any source
- * supported by libdvdread can be given with "path". Currently,
- * libdvdread can access: DVD drives, DVD image files, DVD file-by-file
- * copies.
+ * Attempts to open the DVD drive at the specified path or using external
+ * seek/read functions (dvdnav_open_stream) and pre-cache the CSS-keys.
+ * libdvdread is used to access the DVD, so any source supported by libdvdread
+ * can be given with "path" or "stream_cb".  Currently, using dvdnav_open,
+ * libdvdread can access : DVD drives, DVD image files, DVD file-by-file
+ * copies. Using dvdnav_open_stream, libdvdread can access any kind of DVD
+ * storage via custom implementation of seek/read functions.
  *
  * The resulting dvdnav_t handle will be written to *dest.
  */
 dvdnav_status_t dvdnav_open(dvdnav_t **dest, const char *path);
+dvdnav_status_t
+dvdnav_open_stream(dvdnav_t **dest, void *stream, dvdnav_stream_cb *stream_cb);
+
+dvdnav_status_t dvdnav_dup(dvdnav_t **dest, dvdnav_t *src);
+dvdnav_status_t dvdnav_free_dup(dvdnav_t * _this);
 
 /*
  * Closes a dvdnav_t previously opened with dvdnav_open(), freeing any
@@ -185,7 +183,7 @@ dvdnav_status_t dvdnav_get_PGC_positioning_flag(dvdnav_t *self, int32_t *pgc_bas
  *********************************************************************/
 
 /*
- * These functions are used to poll the playback enginge and actually get data
+ * These functions are used to poll the playback engine and actually get data
  * off the DVD.
  */
 
@@ -204,18 +202,18 @@ dvdnav_status_t dvdnav_get_PGC_positioning_flag(dvdnav_t *self, int32_t *pgc_bas
  * See the dvdnav_events.h header for information on the various events.
  */
 dvdnav_status_t dvdnav_get_next_block(dvdnav_t *self, uint8_t *buf,
-				      int32_t *event, int32_t *len);
+                                      int32_t *event, int32_t *len);
 
 /*
  * This basically does the same as dvdnav_get_next_block. The only difference is
  * that it avoids a memcopy, when the requested block was found in the cache.
- * I such a case (cache hit) this function will return a different pointer than
+ * In such a case (cache hit) this function will return a different pointer than
  * the one handed in, pointing directly into the relevant block in the cache.
  * Those pointers must _never_ be freed but instead returned to the library via
  * dvdnav_free_cache_block().
  */
 dvdnav_status_t dvdnav_get_next_cache_block(dvdnav_t *self, uint8_t **buf,
-					    int32_t *event, int32_t *len);
+                                            int32_t *event, int32_t *len);
 
 /*
  * All buffers which came from the internal cache (when dvdnav_get_next_cache_block()
@@ -283,6 +281,11 @@ dvdnav_status_t dvdnav_title_play(dvdnav_t *self, int32_t title);
  */
 dvdnav_status_t dvdnav_part_play(dvdnav_t *self, int32_t title, int32_t part);
 
+/*
+ * Plays the specified title, starting from the specified program
+ */
+dvdnav_status_t dvdnav_program_play(dvdnav_t *self, int32_t title, int32_t pgcn, int32_t pgn);
+
 /*
  * Stores in *times an array (that the application *must* free) of
  * dvdtimes corresponding to the chapter times for the chosen title.
@@ -299,7 +302,7 @@ uint32_t dvdnav_describe_title_chapters(dvdnav_t *self, int32_t title, uint64_t
  * Currently unimplemented!
  */
 dvdnav_status_t dvdnav_part_play_auto_stop(dvdnav_t *self, int32_t title,
-					   int32_t part, int32_t parts_to_play);
+                                           int32_t part, int32_t parts_to_play);
 
 /*
  * Play the specified title starting from the specified time.
@@ -307,7 +310,7 @@ dvdnav_status_t dvdnav_part_play_auto_stop(dvdnav_t *self, int32_t title,
  * Currently unimplemented!
  */
 dvdnav_status_t dvdnav_time_play(dvdnav_t *self, int32_t title,
-				 uint64_t time);
+                                 uint64_t time);
 
 /*
  * Stop playing the current position and jump to the specified menu.
@@ -318,11 +321,18 @@ dvdnav_status_t dvdnav_menu_call(dvdnav_t *self, DVDMenuID_t menu);
 
 /*
  * Return the title number and part currently being played.
- * A title of 0 indicates, we are in a menu. In this case, part
+ * A title of 0 indicates we are in a menu. In this case, part
  * is set to the current menu's ID.
  */
 dvdnav_status_t dvdnav_current_title_info(dvdnav_t *self, int32_t *title,
-					  int32_t *part);
+                                          int32_t *part);
+
+/*
+ * Return the title number, pgcn and pgn currently being played.
+ * A title of 0 indicates, we are in a menu.
+ */
+dvdnav_status_t dvdnav_current_title_program(dvdnav_t *self, int32_t *title,
+                                          int32_t *pgcn, int32_t *pgn);
 
 /*
  * Return the current position (in blocks) within the current
@@ -332,8 +342,8 @@ dvdnav_status_t dvdnav_current_title_info(dvdnav_t *self, int32_t *title,
  * Use is discouraged!
  */
 dvdnav_status_t dvdnav_get_position_in_title(dvdnav_t *self,
-					     uint32_t *pos,
-					     uint32_t *len);
+                                             uint32_t *pos,
+                                             uint32_t *len);
 
 /*
  * This function is only available for compatibility reasons.
@@ -353,7 +363,7 @@ dvdnav_status_t dvdnav_part_search(dvdnav_t *self, int32_t part);
  * VOBU boundary before the given sector. The sector number is not
  * meant to be an absolute physical DVD sector, but a relative sector
  * in the current program. This function cannot leave the current
- * program and will fail, if asked to do so.
+ * program and will fail if asked to do so.
  *
  * If program chain based positioning is enabled
  * (see dvdnav_set_PGC_positioning_flag()), this will seek to the relative
@@ -363,7 +373,7 @@ dvdnav_status_t dvdnav_part_search(dvdnav_t *self, int32_t part);
  * fcntl.h.
  */
 dvdnav_status_t dvdnav_sector_search(dvdnav_t *self,
-				     uint64_t offset, int32_t origin);
+                                     int64_t offset, int32_t origin);
 
 /*
  returns the current stream time in PTS ticks as reported by the IFO structures
@@ -375,14 +385,15 @@ int64_t dvdnav_get_current_time(dvdnav_t *self);
  * Stop playing the current position and start playback of the title
  * from the specified timecode.
  *
- * Currently unimplemented!
+ * Currently implemented using interpolation. That interpolation is slightly
+ * inaccurate.
  */
 dvdnav_status_t dvdnav_time_search(dvdnav_t *self,
-				   uint64_t time);
+                                   uint64_t time);
 
 /*
  * Stop playing current position and play the "GoUp"-program chain.
- * (which generally leads to the title menu or a higer-level menu).
+ * (which generally leads to the title menu or a higher-level menu).
  */
 dvdnav_status_t dvdnav_go_up(dvdnav_t *self);
 
@@ -413,7 +424,7 @@ dvdnav_status_t dvdnav_next_pg_search(dvdnav_t *self);
  * relative position in and the length of the current program chain.
  */
 dvdnav_status_t dvdnav_get_position(dvdnav_t *self, uint32_t *pos,
-				    uint32_t *len);
+                                    uint32_t *len);
 
 
 /*********************************************************************
@@ -422,12 +433,12 @@ dvdnav_status_t dvdnav_get_position(dvdnav_t *self, uint32_t *pos,
 
 /*
  * Most functions related to highlights take a NAV PCI packet as a parameter.
- * While you can get the such a packet from libdvdnav, for players with internal
- * FIFOs, this will result in errors, because due to the FIFO length, libdvdnav will
- * be ahead in the stream compared to what the user is seeing on screen.
- * Therefore, player applications who have a NAV packet available, which is
- * better in sync with the actual playback should always pass this one to these
- * functions.
+ * While you can get such a packet from libdvdnav, this will result in
+ * errors for players with internal FIFOs because due to the FIFO length,
+ * libdvdnav will be ahead in the stream compared to what the user is
+ * seeing on screen.  Therefore, player applications who have a NAV
+ * packet available, which is better in sync with the actual playback,
+ * should always pass this one to these functions.
  */
 
 /*
@@ -458,7 +469,7 @@ dsi_t* dvdnav_get_current_nav_dsi(dvdnav_t *self);
  * Get the area associated with a certain button.
  */
 dvdnav_status_t dvdnav_get_highlight_area(pci_t *nav_pci , int32_t button, int32_t mode,
-					  dvdnav_highlight_area_t *highlight);
+                                          dvdnav_highlight_area_t *highlight);
 
 /*
  * Move button highlight around as suggested by function name (e.g. with arrow keys).
@@ -484,7 +495,7 @@ dvdnav_status_t dvdnav_button_select(dvdnav_t *self, pci_t *pci, int32_t button)
 dvdnav_status_t dvdnav_button_select_and_activate(dvdnav_t *self, pci_t *pci, int32_t button);
 
 /*
- * Activate (press) a button and execute specified command.
+ * Activate ("press") a button and execute specified command.
  */
 dvdnav_status_t dvdnav_button_activate_cmd(dvdnav_t *self, int32_t button, vm_cmd_t *cmd);
 
@@ -512,19 +523,19 @@ dvdnav_status_t dvdnav_mouse_activate(dvdnav_t *self, pci_t *pci, int32_t x, int
  * Set which menu language we should use per default.
  */
 dvdnav_status_t dvdnav_menu_language_select(dvdnav_t *self,
-					   char *code);
+                                           char *code);
 
 /*
  * Set which audio language we should use per default.
  */
 dvdnav_status_t dvdnav_audio_language_select(dvdnav_t *self,
-					    char *code);
+                                            char *code);
 
 /*
  * Set which spu language we should use per default.
  */
 dvdnav_status_t dvdnav_spu_language_select(dvdnav_t *self,
-					  char *code);
+                                          char *code);
 
 
 /*********************************************************************
@@ -534,13 +545,20 @@ dvdnav_status_t dvdnav_spu_language_select(dvdnav_t *self,
 /*
  * Return a string describing the title of the DVD.
  * This is an ID string encoded on the disc by the author. In many cases
- * this is a descriptive string such as `THE_MATRIX' but sometimes is sigularly
+ * this is a descriptive string such as `THE_MATRIX' but sometimes is singularly
  * uninformative such as `PDVD-011421'. Some DVD authors even forget to set this,
  * so you may also read the default of the authoring software they used, like
  * `DVDVolume'.
  */
 dvdnav_status_t dvdnav_get_title_string(dvdnav_t *self, const char **title_str);
 
+/*
+ * Returns a string containing the serial number of the DVD.
+ * This has a max of 15 characters and should be more unique than the
+ * title string.
+ */
+dvdnav_status_t dvdnav_get_serial_string(dvdnav_t *self, const char **serial_str);
+
 /*
  * Get video aspect code.
  * The aspect code does only change on VTS boundaries.
@@ -550,6 +568,11 @@ dvdnav_status_t dvdnav_get_title_string(dvdnav_t *self, const char **title_str);
  */
 uint8_t dvdnav_get_video_aspect(dvdnav_t *self);
 
+/*
+ * Get video resolution.
+ */
+dvdnav_status_t dvdnav_get_video_resolution(dvdnav_t *self, uint32_t *width, uint32_t *height);
+
 /*
  * Get video scaling permissions.
  * The scaling permission does only change on VTS boundaries.
@@ -572,7 +595,7 @@ uint16_t dvdnav_audio_stream_to_lang(dvdnav_t *self, uint8_t stream);
 uint16_t dvdnav_audio_stream_format(dvdnav_t *self, uint8_t stream);
 
 /*
- * Returns number of channelsn in *logical* audio stream 'stream'
+ * Returns number of channels in *logical* audio stream 'stream'
  * (returns 0xffff if no such stream).
  */
 uint16_t dvdnav_audio_stream_channels(dvdnav_t *self, uint8_t stream);
@@ -641,7 +664,7 @@ user_ops_t dvdnav_get_restrictions(dvdnav_t *self);
  */
 
 /*
- * Sets the current angle. If you try to follow a non existant angle
+ * Sets the current angle. If you try to follow a non existent angle
  * the call fails.
  */
 dvdnav_status_t dvdnav_angle_change(dvdnav_t *self, int32_t angle);
@@ -650,7 +673,7 @@ dvdnav_status_t dvdnav_angle_change(dvdnav_t *self, int32_t angle);
  * Returns the current angle and number of angles present.
  */
 dvdnav_status_t dvdnav_get_angle_info(dvdnav_t *self, int32_t *current_angle,
-				      int32_t *number_of_angles);
+                                      int32_t *number_of_angles);
 
 /*********************************************************************
  * domain queries                                                    *
@@ -681,4 +704,4 @@ int8_t dvdnav_is_domain_vts(dvdnav_t *self);
 }
 #endif
 
-#endif /* DVDNAV_H_INCLUDED */
+#endif /* LIBDVDNAV_DVDNAV_H */
